/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  utility.h - Declaration file for utility functions and global variables
 *  DRV884x_EVM_FIRMWARE
 *  08/25/2022
 *  Jacob Thompson
 *
 ******************************************************************************/
#ifndef UTILITY_H_
#define UTILITY_H_
/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "msp430.h"
#include "dataTypeDefinition.h"

/********************************************************************************
 * FIRMWARE VERSION INFORMATION
 *******************************************************************************/
#define FIRMWARE_VERSION 1.00               // Release version for production EVMs

/******************************************************************************
 * MACROS
 *****************************************************************************/
// GPIO Port 1 Definitions
#define PHASE1       BIT2           // P1.2:  PHASE1 for DRV8845
#define I01          BIT3           // P1.3:  I01 and I11 for DRV8845 (they're tied together)
#define PHASE2       BIT4           // P1.4:  PHASE2 for DRV8845 
#define I02          BIT5           // P1.5:  I02 and I12 for DRV8845 (they're tied together)

#define STEP1        BIT2           // P1.2:  STEP1  for DRV8849
#define DIR1         BIT3           // P1.3:  DIR1   for DRV8849
#define DECAY2       BIT4           // P1.4:  DECAY2 for DRV8849
#define MODE02       BIT5           // P1.5:  MODE02 for DRV8849

// GPIO Port 2 Definitions
#define TP1         BIT2            // P2.2: Test point 1
#define TP2         BIT3            // P2.3: Test point 2
#define TP3         BIT4            // P2.4: Test point 3

#define nFAULT1     BIT0            // P2.0: nFAULT1 on DRV8849
#define nFAULT2     BIT1            // P2.1: nFAULT2 on DRV8849
#define MODE11      BIT5            // P2.5: MODE11  on DRV8849
#define MODE11_PD   BIT6            // P2.6: MODE11_PullDown on DRV8849
#define DECAY1      BIT7            // P2.7: DECAY1  on DRV8849

// GPIO Port 3 Definitions
#define TP6                 BIT0    // P3.0: Test point 6
#define MCU_STATUS          BIT1    // P3.1: MCU Status LED output
#define GUI_STATUS          BIT2    // P3.2: GUI Status LED output
#define nSLEEP              BIT7    // P3.7: nSLEEP control (both boards)

#define ENABLE1_STATUS      BIT3    // P3.3: ENABLE1 status LED on DRV8849
#define ENABLE2_STATUS      BIT4    // P3.4: ENABLE2 status LED on DRV8849
#define MODE01              BIT5    // P3.5: MODE01  on DRV8849
#define ENABLE1             BIT6    // P3.6: ENABLE1 on DRV8849

// GPIO Port 4 Definitions
#define PHASE3              BIT1    // P4.1: PHASE3 on DRV8845
#define I03                 BIT2    // P4.2: I03 and I13 on DRV8845
#define PHASE4              BIT3    // P4.3: PHASE4 on DRV8845
#define I04                 BIT4    // P4.4: I04 and I14 on DRV885

#define STEP2               BIT1    // P4.1: STEP2   on DRV8849
#define DIR2                BIT2    // P4.2: DIR2    on DRV8849
#define ENABLE2             BIT3    // P4.3: ENABLE2 on DRV8849
#define MODE12              BIT4    // P4.4: MODE12  on DRV8849
#define MODE12_PD           BIT5    // P4.5: MODE12_PullDown on DRV8849
#define DECAY1_PD           BIT6    // P4.6: DECAY1_PullDown on DRV8849
#define DECAY2_PD           BIT7    // P4.7: DECAY2_PullDown on DRV8849

// GPIO Port 5 Definitions
#define IDR_1               BIT3    // P5.3: EVM ID1 setting
#define IDR_2               BIT4    // P5.4: EVM ID2 setting
#define IDR_3               BIT5    // P5.5: EVM ID3 setting
#define TP4                 BIT6    // P5.6: Test point 4
#define TP5                 BIT7    // P5.6: Test point 5

// GPIO Port 6 Definitions
#define VSEN_VM             BIT0    // P6.0: Voltage Input Sensing
#define VREF1_Pin           BIT6    // P6.6: VREF1 for DRV8849, VREF1 and VREF3 for DRV8845
#define VREF2_Pin           BIT7    // P6.7: VREF2 for DRV8849, VREF2 and VREF4 for DRV8845

#define AVREF_DAC	          DAC12_0DAT  // P6.6 - DAC0, VREF1 and VREF3
#define BVREF_DAC	          DAC12_1DAT  // P6.7 - DAC1, VREF2 and VREF4

// GPIO Port 8 Definitions
#define MODE02_PD           BIT0    // P8.0:  MODE02_PullDown for DRV8849
#define MODE01_PD           BIT1    // P8.1:  MODE01_PullDown for DRV8849
#define UART_TXD            BIT2    // P8.2
#define UART_RXD            BIT3    // P8.3

// nSLEEP GPIO macro for 8849
#define set_nSLEEP_hi             P3OUT |= nSLEEP
#define set_nSLEEP_lo             P3OUT &= ~nSLEEP

// ENABLE GPIO macro for 8849
#define set_ENABLE1_hi            P3OUT |=  ENABLE1
#define set_ENABLE1_lo            P3OUT &= ~ENABLE1
#define set_ENABLE2_hi            P4OUT |=  ENABLE2
#define set_ENABLE2_lo            P4OUT &= ~ENABLE2

// DIR GPIO macro for 8849
#define set_DIR1_hi               P1OUT |= DIR1
#define set_DIR1_lo               P1OUT &= ~DIR1
#define set_DIR2_hi               P4OUT |= DIR2
#define set_DIR2_lo               P4OUT &= ~DIR2

// STEP GPIO macro
#define set_STEP1_hi              P1OUT |= STEP1
#define set_STEP1_lo              P1OUT &= ~STEP1
#define set_STEP2_hi              P4OUT |= STEP2
#define set_STEP2_lo              P4OUT &= ~STEP2

#define read_nSLEEP_pin           P3IN & nSLEEP
#define read_ENABLE1_pin          P3IN & ENABLE1
#define read_ENABLE2_pin          P4IN & ENABLE2
#define read_nFAULT1_pin          P2IN & nFAULT1
#define read_nFAULT2_pin          P2IN & nFAULT2
#define read_MODE01_pin           P3IN & MODE01
#define read_MODE02_pin           P1IN & MODE02
#define read_MODE11_pin           P2IN & MODE11
#define read_MODE12_pin           P4IN & MODE12

// LEDs 
#define toggle_status_led         P3OUT ^= GUI_STATUS
#define toggle_MCUstatus_led      P3OUT ^= MCU_STATUS
#define set_ENABLE1_led_hi        P3OUT |= ENABLE1_STATUS
#define set_ENABLE1_led_lo        P3OUT &= ~ENABLE1_STATUS
#define set_ENABLE2_led_hi        P3OUT |= ENABLE2_STATUS
#define set_ENABLE2_led_lo        P3OUT &= ~ENABLE2_STATUS
#define set_TP_debug_timing_hi    P5OUT |= TP5
#define set_TP_debug_timing_lo    P5OUT &= ~TP5
#define set_TP1_debug_timing_hi   P2OUT |= TP1
#define set_TP1_debug_timing_lo   P2OUT &= ~TP1

#define read_ID1_pin              P5IN & IDR_1
#define read_ID2_pin              P5IN & IDR_2
#define read_ID3_pin              P5IN & IDR_3

// DECAY GPIO macros (DRV8849 only)
// Decay 1 (DRV8849 only)
#define enable_DECAY1_output      P2DIR |=  DECAY1
#define disable_DECAY1_output     P2DIR &= ~DECAY1
#define enable_DECAY1_PD_output   P4DIR |=  DECAY1_PD
#define disable_DECAY1_PD_output  P4DIR &= ~DECAY1_PD
#define set_DECAY1_lo             P2OUT &= ~DECAY1
#define set_DECAY1_hi             P2OUT |= DECAY1
#define set_DECAY1_PD_lo          P4OUT &= ~DECAY1_PD
#define set_DECAY1_PD_hi          P4OUT |= DECAY1_PD

// Decay 2 (DRV8849 only)
#define enable_DECAY2_output      P1DIR |=  DECAY2
#define disable_DECAY2_output     P1DIR &= ~DECAY2
#define enable_DECAY2_PD_output   P4DIR |=  DECAY2_PD
#define disable_DECAY2_PD_output  P4DIR &= ~DECAY2_PD
#define set_DECAY2_lo             P1OUT &= ~DECAY2
#define set_DECAY2_hi             P1OUT |= DECAY2
#define set_DECAY2_PD_lo          P4OUT &= ~DECAY2_PD
#define set_DECAY2_PD_hi          P4OUT |= DECAY2_PD

// Decay mode (DRV8849 only)
#define SMART_TUNE        0       // Smart Tune Ripple Control is DECAYx pin LOW
#define STDD_7us          1       // Smart Tune Dynamic Decay 32us is DECAYx pin Hi-Z
#define STDD_16us         2       // Smart Tune Dynamic Decay 16us is DECAYx pin HIGH
#define STDD_32us         3       // Smart Tune Dynamic Decay 7us is  DECAYx pin 330K to GND


#define set_DECAY1_SmartTune   set_DECAY1_lo;     enable_DECAY1_output;     disable_DECAY1_PD_output;
#define set_DECAY1_DD32   disable_DECAY1_output;  disable_DECAY1_PD_output;
#define set_DECAY1_DD16   set_DECAY1_hi;          enable_DECAY1_output;     disable_DECAY1_PD_output;
#define set_DECAY1_DD7    disable_DECAY1_output;  set_DECAY1_PD_lo;         enable_DECAY1_PD_output;

#define set_DECAY2_SmartTune   set_DECAY2_lo;     enable_DECAY2_output;     disable_DECAY2_PD_output;
#define set_DECAY2_DD32   disable_DECAY2_output;  disable_DECAY2_PD_output;
#define set_DECAY2_DD16   set_DECAY2_hi;          enable_DECAY2_output;     disable_DECAY2_PD_output;
#define set_DECAY2_DD7    disable_DECAY2_output;  set_DECAY2_PD_lo;         enable_DECAY2_PD_output;


// Step Mode (DRV8849 only)
#define FULLSTEP_100       0       // Full step (2-phase excitation) with 100% current
#define FULLSTEP_71        1       // Full step (2-phase excitation) with 71% current
#define NONCIRCLR_uSTEP_2  2       // Non-circular 1/2 step
#define uSTEP_2            3       // 1/2 step
#define uSTEP_4            4       // 1/4 step
#define uSTEP_8            5       // 1/8 step
#define uSTEP_16           6       // 1/16 step
#define uSTEP_32           7       // 1/32 step
#define uSTEP_64           8       // 1/64 step
#define uSTEP_128          9       // 1/128 step
#define uSTEP_256          10      // 1/256 step

// Set Step Mode - Stepper 1 (DRV8849 only)
#define set_MODE01_hi       P8DIR &= ~MODE01_PD;  P3DIR |= MODE01;  P3OUT |= MODE01  
#define set_MODE01_lo       P8DIR &= ~MODE01_PD;  P3DIR |= MODE01;  P3OUT &= ~MODE01 
#define set_MODE01_hiz      P8DIR &= ~MODE01_PD;  P3DIR &= ~MODE01

#define set_MODE11_hi       P2DIR &= ~MODE11_PD;  P2DIR |= MODE11;  P2OUT |= MODE11  
#define set_MODE11_lo       P2DIR &= ~MODE11_PD;  P2DIR |= MODE11;  P2OUT &= ~MODE11 
#define set_MODE11_hiz      P2DIR &= ~MODE11_PD;  P2DIR &= ~MODE11;
#define set_MODE11_330k     P2DIR |= MODE11_PD;   P2DIR &= ~MODE11; P2OUT &= ~MODE11_PD

#define set_fullstep_100_Stepper1          set_MODE01_lo;  set_MODE02_lo
#define set_fullstep_71_Stepper1           set_MODE01_lo;  set_MODE02_330k
#define set_noncircular_halfstep_Stepper1  set_MODE01_hi;  set_MODE02_lo
#define set_halfstep_Stepper1              set_MODE01_hiz; set_MODE02_lo
#define set_microstep_4_Stepper1           set_MODE01_lo;  set_MODE02_hi
#define set_microstep_8_Stepper1           set_MODE01_hi;  set_MODE02_hi
#define set_microstep_16_Stepper1          set_MODE01_hiz; set_MODE02_hi
#define set_microstep_32_Stepper1          set_MODE01_lo;  set_MODE02_hiz
#define set_microstep_64_Stepper1          set_MODE01_hiz; set_MODE02_330k
#define set_microstep_128_Stepper1         set_MODE01_hiz; set_MODE02_hiz
#define set_microstep_256_Stepper1         set_MODE01_hi;  set_MODE02_hiz

// Set Step Mode - Stepper 2 (DRV8849 only)
//                       #Set Mode02_PD as Hi-Z#  # Set as OUTPUT#    # Set HIGH/LOW #
#define set_MODE02_hi       P8DIR &= ~MODE02_PD;  P1DIR |= MODE02;    P1OUT |= MODE02    
#define set_MODE02_lo       P8DIR &= ~MODE02_PD;  P1DIR |= MODE02;    P1OUT &= ~MODE02  
#define set_MODE02_hiz      P8DIR &= ~MODE02_PD;  P1DIR &= ~MODE02     
#define set_MODE02_330k     P8DIR |= MODE02_PD;   P1DIR &= ~MODE02;   P1OUT &= ~MODE02_PD
#define set_MODE12_hi       P4DIR &= ~MODE12_PD;  P4DIR |= MODE12;    P4OUT |= MODE12  
#define set_MODE12_lo       P4DIR &= ~MODE12_PD;  P4DIR |= MODE12;    P4OUT &= ~MODE12
#define set_MODE12_hiz      P4DIR &= ~MODE12_PD;  P4DIR &= ~MODE12
#define set_MODE12_330k     P4DIR |= MODE12_PD;   P4DIR &= ~MODE12;   P4OUT &= ~MODE12_PD   

#define set_fullstep_100_Stepper2          set_MODE11_lo;  set_MODE12_lo
#define set_fullstep_71_Stepper2           set_MODE11_lo;  set_MODE12_330k
#define set_noncircular_halfstep_Stepper2  set_MODE11_hi;  set_MODE12_lo
#define set_halfstep_Stepper2              set_MODE11_hiz; set_MODE12_lo
#define set_microstep_4_Stepper2           set_MODE11_lo;  set_MODE12_hi
#define set_microstep_8_Stepper2           set_MODE11_hi;  set_MODE12_hi
#define set_microstep_16_Stepper2          set_MODE11_hiz; set_MODE12_hi
#define set_microstep_32_Stepper2          set_MODE11_lo;  set_MODE12_hiz
#define set_microstep_64_Stepper2          set_MODE11_hiz; set_MODE12_330k
#define set_microstep_128_Stepper2         set_MODE11_hiz; set_MODE12_hiz
#define set_microstep_256_Stepper2         set_MODE11_hi;  set_MODE12_hiz

// Set or Read ENABLE pins (DRV8849 only)
#define read_ENABLE1_pin              P3IN  & ENABLE1
#define set_ENABLE1_lo                P3OUT &= ~ENABLE1
#define set_ENABLE1_hi                P3OUT |= ENABLE1
#define read_ENABLE2_pin              P4IN  & ENABLE2
#define set_ENABLE2_lo                P4OUT &= ~ENABLE2
#define set_ENABLE2_hi                P4OUT |= ENABLE2

#define _PWM_TIMER_PERIOD 100
#define PWM_TIMER_PERIOD (_PWM_TIMER_PERIOD-1)
#define PWM_SCALE (_PWM_TIMER_PERIOD/100)
#define PWM_FREQ_100KHZ           ID0
#define PWM_FREQ_50KHZ            ID1
#define PWM_FREQ_25KHZ            ID1 | ID0

#define TA1CCR1_DEFAULT           2
#define NumSamples                48
#define ADC12scale                68


#define REV_PG1_0          0
#define REV_PG2_0          1

/******************************************************************************
* DATA STRUCTURES
 *****************************************************************************/
typedef struct DRV884x_Device_Obj_t
{
  float Firmware_Version;           // Firmware version, see readme.txt for details (4 bit)
  volatile uint16_t   VMvoltage;    // VM supply voltage

  // Device Parameters
  volatile uint16_t deviceID;         // IC_ID set value
  volatile uint8_t revID;             // Silicon revision ID either from Register read or set value
  volatile uint8_t nSleep;            // 0: Device Sleep , 1: Device Awake
  volatile uint8_t gDRVEN1;           // 0: Outputs Hi-Z, 1: Outputs active for Stepper 1
  volatile uint8_t gDRVEN2;           // 0: Outputs Hi-Z, 1: Outputs active for Stepper 2
  volatile uint8_t nFault1;           // 0: No fault , 1: Fault for Stepper 1
  volatile uint8_t nFault2;           // 0: No fault , 1: Fault for Stepper 2
  volatile uint8_t ClrFlt;            // 0: Button not pressed, 1: Button pressed
  volatile uint8_t MotorMode;         // Brushed or Stepper - 0: Stepper Mode, 1: Brushed Mode (DRV8845)
  volatile uint8_t MotorModePrevious; // 0: Stepper Mode, 1: Brushed Mode (DRV8845 ONLY)
  volatile uint8_t StepMode1;         // GUI Microstep Mode settings for Stepper 1
  volatile uint8_t StepMode2;         // GUI Microstep Mode settings for Stepper 2
  volatile uint8_t DecayMode1;        // GUI Decay Mode settings for Stepper 1 (DRV8849 ONLY)
  volatile uint8_t DecayMode2;        // GUI Decay Mode settings for Stepper 2 (DRV8849 ONLY)
} DRV884x_Device_Obj_t;


/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
extern volatile DRV884x_Device_Obj_t gDRV884x_Obj;

/******************************************************************************
 * API Prototypes
 *****************************************************************************/
void getEVMID(void);
void drv884x_init(void);
void SetDefaultGUIVals(void);
void ClearFaultPulse(void);
void mcu_init(void);

/*****************************************************************************/
#endif // UTILITY_H_
