/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  stepper.h - Declaration file for functions and global variables related to stepper control
 *  DRV8845_EVM_FIRMWARE
 *  08/25/2022
 *  Jacob Thompson
 *  03/01/2023
 *  Murugavel Raju
 *
 ******************************************************************************/
#ifndef STEPPER_8845_H_
#define STEPPER_8845_H_
/*******************************************************************************
 * HEADER FILES
 ******************************************************************************/
#include <dataTypeDefinition.h>
#include "drv884x/include/drv8845.h"
#include "drv884x/include/utility.h"
#include <msp430.h>
#include <stepper.h>

/*******************************************************************************
 * Defined Quantities
 ******************************************************************************/
#define LED_TIMEOUT 125                                    // Count to blink LED with 1s period
#define SMCLK_8845  2500000                                // SMCLK frequency 2.5 MHz
#define TIME_32ms45   40001                                // TimerA interrupt value, (32ms or 31.25 updates/s) at 1.25 MHz

// Stepper 1 and 2 default settings
#define DEFAULT_STEP_MODE_8845	 5	                       // 1/16 uStep
#define DEFAULT_CURRENT_SETTING  11	                       // DRV8845 current setting with 1.5V MCU_VREF value of 11 = 0.5A
#define use_one_edge             0                         // multiplies all step calculations by 1
#define Lower_12_bits            0xFFF

// defines for motor state machine
#define IDLE            1
#define ACCEL           2
#define DECEL           4
#define RUN             8
#define DECEL2NEWTARGET 16

#define		ACCEL_RATE_COUNT 2000


// variables for the motor state machine
unsigned int AccelerateState;
unsigned int StepMode;
unsigned long int SteppingRateTMR;
unsigned long int SteppingRate;
unsigned long int SteppingRateUpdate;
unsigned long int DesiredStepperSpeed;
unsigned long int DesiredTargetSpeed;
unsigned long int StartingSpeed;
unsigned long int StoppingSpeed;
unsigned long int AccelRate;
unsigned long int AccelTimeBase;
unsigned long int AccelerationIncrease;
unsigned long int StepsToStop;
unsigned long int StepsToMove;
unsigned long int tmpStepsToMove;
unsigned long int PriorSteppingRate;
unsigned long int Steps2Target;                            // number of steps to reach target speed from starting speed using acceleration rate
unsigned long int Steps2Stop;                              // number of steps to stop from target speed using acceleration rate
unsigned long int Steps2Decel;                             // step number to change from accel to decel in manual mode
unsigned int last_reciprocation_state;                     // reciprocation flag, =1 last action is reciprocation
unsigned int last_reciprocation_state2;                    // reciprocation flag, =1 last action is reciprocation

unsigned int LED_TOGGLE_CNT;                               // count to determine if LED should be toggled
unsigned int clear_fault_loop;                             // counter for clear fault pulse

bool Reciprocate;
bool ManualAdvance;
bool temp_reciprocation;
bool LastCommandWasRunSteps;
bool Direction;
bool MoveSteps;
bool RunStepper;
bool UpdateSteppingRateTMR;

unsigned char next_Direction;
unsigned char ichop_val;
unsigned char tmp_ichop_val;
unsigned char temp_step_mode;
unsigned char prev_gDRVEN1;
unsigned char prev_gDRVEN2;

float time;
float time_squared;

// Stepper indexer variables for DRV8845
unsigned int move_to_next_step;
unsigned int step_offset;
unsigned int current_scale;
unsigned int Next_a_step;
unsigned int Next_b_step;
unsigned int Next_a_sine_table;
unsigned int Next_b_sine_table;
unsigned int Next_a_value;
unsigned int Next_b_value;
unsigned int Next_eighth_a;
unsigned int Next_eighth_b;
unsigned int Next_full_a;
unsigned int Next_full_b;
unsigned int Next_half_a;
unsigned int Next_half_b;
unsigned int Next_quarter_a;
unsigned int Next_quarter_b;
unsigned int Next_sixteenth_a;
unsigned int Next_sixteenth_b;
unsigned int Next_Non_CirA;
unsigned int Next_Non_CirB;
unsigned int Next_PHA;
unsigned int Next_PHB;
unsigned int Next_value_calculated;

/******************************************************************************/
// Function Declarations

// Main Functions
void UpdateDeviceControls_Stepper_DRV8845();
void UpdateGUIControls_DRV8845();
void UpdateSpeed_8845();

// GPIO/GUI Functions
void stepper_8845_Init();

// Motor Functions
void DetermineMotorMovement_8845();
void MotorStateMachine_8845();

#endif /*STEPPER_H_*/
